/*jslint vars: true, nomen: true, plusplus: true, eqeq: true */
/*globals console */
/*globals SceneManager, Scene_MenuBase, Window_Selectable, Window_MenuCommand, Window_ItemList, Window_Base, Window_Command */
/*globals DataManager, Game_Interpreter, PluginManager, Graphics, TouchInput */
/*globals $gameSwitches, $gameVariables, $gameParty */
//=============================================================================
// Menu.js
//=============================================================================

/*:
 * @plugindesc Pivert Crafting system.
 * @author Pivert
 *
 * @param Min items
 * @desc Min number of items used for crafting
 * @default 3
 *
 * @param Max items
 * @desc Max number of items used for crafting
 * @default 5
 *
 * @param Title message
 * @desc Title message
 * @default Select 3 to 5 items to craft
 *
 * @param Min items message
 * @desc Message displayed if not enough items are selected
 * @default Select 3 to 5 items to craft
 *
 * @param Failure message
 * @desc Message displayed if recipe fail
 * @default Recipe failed
 *
 * @param Already crafted message
 * @desc Message displayed if recipe was already crafted
 * @default Recipe already crafted
 *
 */
(function () {

    'use strict';

    var crafting = [];

    var parameters = PluginManager.parameters('Crafting');
    var minItems = Number(parameters['Min items'] || 3);
    var maxItems = Number(parameters['Max items'] || 5);
    var titleMessage = String(parameters['Title message'] || 'Select 3 to 5 items to craft');
    var minItemsMessage = String(parameters['Min items message'] || 'Select 3 to 5 items to craft');
    var failureMessage = String(parameters['Failure message'] || 'Recipe failed');
    var alreadyCraftedMessage = String(parameters['Already crafted message'] || 'Recipe already crafted');

    //=============================================================================
    // Crafting list window
    //=============================================================================

    function Window_CraftingItemList() {
        this.initialize.apply(this, arguments);
    }

    Window_CraftingItemList.prototype = Object.create(Window_ItemList.prototype);
    Window_CraftingItemList.prototype.constructor = Window_CraftingItemList;

    Window_CraftingItemList.prototype.initialize = function (x, y, width, height) {
        Window_ItemList.prototype.initialize.call(this, x, y, width, height);
        this.makeItemList();
        this.refresh();
        this.contents.fillRect(0, this.lineHeight() * 3 / 2, this.contentsWidth(), 2, this.normalColor());
    };

    Window_CraftingItemList.prototype.isCursorVisible = function () {
        var row = this.row();
        var item = this.item();
        return row >= this.topRow() && row <= this.bottomRow() && item.enabled;
    };

    Window_CraftingItemList.prototype.makeItemList = function () {
        this._data = $gameParty.allItems().filter(function (item) {
            item.selectedForCraft = false;
            item.enabled = true;
            return this.includes(item);
        }, this);
        
        if (typeof SSG_Heartbreak.itemSort === 'function'){
            SSG_Heartbreak.itemSort(this._data);
        }
    
        if (this.includes(null)) {
            this._data.push(null);
        }
        this._data.unshift({
            name: '',
            isCommand: false,
            enabled: false
        });
        this._data.unshift({
            name: '',
            isCommand: false,
            enabled: false
        });
        this._data.unshift({
            name: '',
            isCommand: false,
            enabled: false
        });
        this._data.unshift({
            name: 'Craft',
            isCommand: true,
            enabled: true
        });
    };

    Window_CraftingItemList.prototype.includes = function (item) {
        return DataManager.isItem(item) && item.itypeId === 2;
    };

    Window_CraftingItemList.prototype.processOk = function () {
		if (!this.item()) {
			this.playBuzzerSound();
		} else if (this.isCurrentItemEnabled() || this.item().isCommand) {
            this.updateInputData();
            this.deactivate();
            this.callOkHandler();
        }
    };

    Window_CraftingItemList.prototype.drawItem = function (index) {
        var item = this._data[index];
        if (item) {
            var rect = this.itemRect(index);
            rect.width -= this.textPadding();
            this.drawItemName(item, rect.x, rect.y, rect.width);
            this.resetTextColor();
        }
    };

    Window_CraftingItemList.prototype.drawItemName = function (item, x, y, width) {
        width = width || 312;
        if (item) {
            var iconBoxWidth = Window_Base._iconWidth + 4;
            this.drawIcon(item.iconIndex, x + 2, y + 2);
            if (item.selectedForCraft) {
                this.changeTextColor(this.textColor(17));
                this.drawText(item.name, x + iconBoxWidth, y, width - iconBoxWidth);
                this.resetTextColor();
            } else {
                this.drawText(item.name, x + iconBoxWidth, y, width - iconBoxWidth);
            }
        }
    };

    Window_CraftingItemList.prototype.refresh = function () {
        this.createContents();
        this.drawAllItems();
        this.contents.fillRect(0, this.lineHeight() * 3 / 2, this.contentsWidth(), 2, this.normalColor());
    };

    //=============================================================================
    // Crafting message window
    //=============================================================================

    function Window_CraftingMessage() {
        this.initialize.apply(this, arguments);
    }

    Window_CraftingMessage.prototype = Object.create(Window_MenuCommand.prototype);
    Window_CraftingMessage.prototype.constructor = Window_CraftingMessage;

    Window_CraftingMessage.prototype.initialize = function () {
        this._visible = false;

        Window_MenuCommand.prototype.initialize.call(this, 0, 0);
        this.height = this.fittingHeight(1);
    };

    Window_CraftingMessage.prototype.windowWidth = function () {
        return Graphics.boxWidth;
    };

    Window_CraftingMessage.prototype.maxCols = function () {
        return 1;
    };

    Window_CraftingMessage.prototype.maxItems = function () {
        return 1;
    };

    Window_CraftingMessage.prototype.setText = function (text) {
        this._text = text;
        this.refresh();
    };

    Window_CraftingMessage.prototype.drawItem = function (index) {
        var rect = this.itemRectForText(index);
        var align = this.itemTextAlign();
        this.resetTextColor();
        this.changePaintOpacity(this.isCommandEnabled(index));
        this.drawText(this._text, rect.x, rect.y, rect.width, align);
    };

    //=============================================================================
    // Crafting scene
    //=============================================================================

    function Scene_Crafting() {
        this.initialize.apply(this, arguments);
    }

    Scene_Crafting.prototype = Object.create(Scene_MenuBase.prototype);
    Scene_Crafting.prototype.constructor = Scene_Crafting;

    Scene_Crafting.prototype.initialize = function () {
        Scene_MenuBase.prototype.initialize.call(this);
        this._selectedItems = [];
    };

    Scene_Crafting.prototype.create = function () {
        Scene_MenuBase.prototype.create.call(this);
        this.createMessageWindow();
        this.createItemWindow();
        this._messageWindow.deselect();
        this._messageWindow.deactivate();
        this._itemWindow.activate();
		this._itemWindow.select(0);
    };

    Scene_Crafting.prototype.createMessageWindow = function () {
        this._messageWindow = new Window_CraftingMessage();
        this._messageWindow.y = Graphics.boxHeight - this._messageWindow.height;
        this.addWindow(this._messageWindow);
        this._messageWindow.setText(titleMessage);
    };

    Scene_Crafting.prototype.createItemWindow = function () {
        var height = Graphics.boxHeight - this._messageWindow.height;
        this._itemWindow = new Window_CraftingItemList(0, 0, Graphics.boxWidth, height);
        this._itemWindow.setHandler('ok', this.onItemOk.bind(this));
        this._itemWindow.setHandler('cancel', this.popScene.bind(this));
        this.addWindow(this._itemWindow);
    };

    Scene_Crafting.prototype.onItemOk = function () {
        var item = this._itemWindow.item();

        if (item.isCommand) {
            this.craft();
        } else {
            if (item.selectedForCraft) {
                var i;
                for (i = 0; i < this._selectedItems.length; i++) {
                    if (this._selectedItems[i].name === item.name) {
                        break;
                    }
                }
                this._selectedItems.splice(i, 1);
                item.selectedForCraft = false;
            } else if (this._selectedItems.length < maxItems) {
                this._selectedItems.push(item);
                item.selectedForCraft = true;
            }

            if (this._selectedItems.length == maxItems) {
                this._itemWindow.select(0);
            }

            this._itemWindow.refresh();
        }

        this._itemWindow.activate();
		this._itemWindow.select(0);
    };

    Scene_Crafting.prototype.craft = function () {

        if (this._selectedItems.length < minItems) {
            this._messageWindow.setText(minItemsMessage);
            this._itemWindow.playBuzzerSound();
            return;
        }
        // Récupérer la recette
        var recipe;
        var i;
        for (i = 0; i < crafting.length; i++) {
            if (this._selectedItems.length !== crafting[i].recipe.length) {
                continue;
            }

            var done = true;

            var j;
            for (j = 0; j < this._selectedItems.length; j++) {
                if (crafting[i].recipe.indexOf(this._selectedItems[j].id) === -1) {
                    done = false;
                    break;
                }
            }

            if (done) {
                recipe = crafting[i];
                break;
            }
        }

        if (!recipe) {
            this._messageWindow.setText(failureMessage);
            this._itemWindow.playBuzzerSound();
            return;
        }

        var success = recipe.success;
        if (typeof success === 'number') {
            success = $gameSwitches.value(success);
        } else if (typeof recipe.success === 'boolean') {
            success = recipe.success;
        }

        if (!success) {
            this._messageWindow.setText(recipe.failText || failureMessage);
            this._itemWindow.playBuzzerSound();
            return;
        }

        var value = $gameVariables.value(recipe.variableId);
        if (value > 0) {
            this._messageWindow.setText(recipe.alreadyCraftedText || alreadyCraftedMessage);
            this._itemWindow.playBuzzerSound();
            return;
        }

        this._itemWindow.playOkSound();
        $gameVariables.setValue(recipe.variableId, value + 1);
        SceneManager.pop();
    };

    //=============================================================================
    // Game interpreter
    //=============================================================================

    var _Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function (command, args) {
        _Game_Interpreter_pluginCommand.call(this, command, args);
        if (command === 'Crafting' && args[0] === 'open') {
            SceneManager.push(Scene_Crafting);
        }
    };

    var loadAchievements = function () {

        var xhr = new XMLHttpRequest();
        xhr.overrideMimeType('application/json');
        xhr.open('GET', 'data/Crafting.json');

        xhr.onload = function () {
            if (xhr.status < 400) {
                try {
                    crafting = JSON.parse(xhr.responseText);
                } catch (e) {
                    console.error('Could not load script');
                    console.error(e);
                }
            } else {
                console.error('File data/Crafting.json not found');
            }
        };

        xhr.onerror = function (e) {
            console.error('Error while fetching data/Crafting.json');
            console.error(e);
        };

        xhr.send();
    };

    loadAchievements();

}());
