//=============================================================================
// Menu.js
//=============================================================================

/*:
 * @plugindesc Pivert Interactive map.
 * @author Pivert
 *
 * @param Images directory
 * @desc The directory where map images are stored
 * @default img/map
 *
 * @param Font size
 * @desc The font size of map texts
 * @default 16
 *
 */

(function () {
    'use strict';

    var parameters = PluginManager.parameters('InteractiveMap');

    var imageDirectory = String(parameters['Images directory'] || 'img/map');
    var fontSize = Number(parameters['Font size'] || 16);

    var TouchInput_checkCursorStatus = TouchInput._checkCursorStatus;

    //=============================================================================
    // Scene_Map
    //=============================================================================
    var Scene_Map_initialize = Scene_Map.prototype.initialize;
    Scene_Map.prototype.initialize = function () {
        Scene_Map_initialize.call(this);
        this._idToShow = null;
        this._idToHide = null;
    };

    var Scene_Map_update = Scene_Map.prototype.update;
    Scene_Map.prototype.update = function () {
        Scene_Map_update.call(this);

        if (this._showMap) {
            var i, x, y, w, h;

            var tX = TouchInput._cursorX;
            var tY = TouchInput._cursorY;

            var noSpot = true;

            for (i = 0; i < $dataInteractiveMap.length; i++) {
                if (this.onSpot(tX, tY, $dataInteractiveMap[i])) {
                    this._idToShow = i;
                    this._idToHide = null;
                    noSpot = false;
                    break;
                }
            }

            if (noSpot) {
                this._idToHide = this._idToShow;
                this._idToShow = null;
            }

            if (this._idToShow !== null) {
                document.body.style.cursor = 'pointer';
                this._texts[$dataInteractiveMap[this._idToShow].id].visible = true;
            }

            if (this._idToHide != null) {
                document.body.style.cursor = 'auto';
                this._texts[$dataInteractiveMap[this._idToHide].id].visible = false;
            }
        }
    };

    var Scene_Map_terminate = Scene_Map.prototype.terminate;
    Scene_Map.prototype.terminate = function () {
        Scene_Map_terminate.call(this);
        TouchInput._checkCursorStatus = TouchInput_checkCursorStatus;
    };

    Scene_Map.prototype.onSpot = function (tX, tY, spot) {
        // Spot is locked
        if (spot.trigger && !$gameSwitches.value(spot.trigger)) {
            return false;
        }

        // Not on spot
        if (tX < spot.x || tX > spot.x + spot.image.width || tY < spot.y || tY > spot.y + spot.image.height) {
            return false;
        }

        if (TouchInput.isTriggered()) {
            var event = $gameMap.event(spot.event);
            if (event) {
                event.start();
            }
        }

        return true;
    };

    Scene_Map.prototype.initSprites = function () {
        var i, x, y, w, h;

        this._sprites = {};
        this._texts = {};

        // Image
        for (i = 0; i < $dataInteractiveMap.length; i++) {
            x = $dataInteractiveMap[i].x;
            y = $dataInteractiveMap[i].y;

            var imageSprite = new Sprite($dataInteractiveMap[i].image);
            imageSprite.x = x;
            imageSprite.y = y;
            imageSprite.visible = !$dataInteractiveMap[i].trigger || $gameSwitches.value($dataInteractiveMap[i].trigger);
            this.addChild(imageSprite);
            this._sprites[$dataInteractiveMap[i].id] = imageSprite;
        }

        // Text
        for (i = 0; i < $dataInteractiveMap.length; i++) {
            x = $dataInteractiveMap[i].x;
            y = $dataInteractiveMap[i].y;
            w = $dataInteractiveMap[i].image.width;
            h = $dataInteractiveMap[i].image.height;

            var offset = Graphics.boxWidth - (x + w) - 5;
            x -= offset;
            w += 2 * offset;
            if ($dataInteractiveMap[i].textPosition === 'top') {
                y -= 40;
            } else if ($dataInteractiveMap[i].textPosition === 'bottom') {
                y += h;
            }

            var bitmap = new Bitmap(w, 50);
            bitmap.fontSize = fontSize;
            bitmap.drawText($dataInteractiveMap[i].name, 0, 6, w, 20, 'center');

            var textSprite = new Sprite(bitmap);
            textSprite.x = x;
            textSprite.y = y;
            textSprite.visible = false;
            this.addChild(textSprite);
            this._texts[$dataInteractiveMap[i].id] = textSprite;
        }
    };

    Scene_Map.prototype.showMap = function () {
        this._showMap = true;
        if (!this._init) {
            this._init = true;
            this.initSprites();
        }
    };

    //=============================================================================
    // TouchInput
    //=============================================================================
    var TouchInput_onMouseMove = TouchInput._onMouseMove;
    TouchInput._onMouseMove = function (event) {
        TouchInput_onMouseMove.call(this, event);

        this._cursorX = Graphics.pageToCanvasX(event.pageX);
        this._cursorY = Graphics.pageToCanvasY(event.pageY);
    };

    //=============================================================================
    // Game_Interpreter
    //=============================================================================
    var _Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function (command, args) {
        _Game_Interpreter_pluginCommand.call(this, command, args);
        if (command === 'InteractiveMap' && args[0] === 'open') {
            TouchInput._checkCursorStatus = function () { };
            SceneManager._scene.showMap();
        }
    };

    //=============================================================================
    // Scene_Boot
    //=============================================================================
    var Scene_Boot_create = Scene_Boot.prototype.create;
    Scene_Boot.prototype.create = function () {
        DataManager._databaseFiles.push({ name: '$dataInteractiveMap', src: 'InteractiveMap.json' });
        Scene_Boot_create.call(this);
    };

    //=============================================================================
    // DataManager
    //=============================================================================
    var DataManager_onLoad = DataManager.onLoad;
    DataManager.onLoad = function (object) {
        DataManager_onLoad.call(this, object);
        if (object === $dataInteractiveMap) {
            $dataInteractiveMap.forEach(function (icon) {
                icon.image = ImageManager.loadBitmap(imageDirectory + '/', icon.image);
            });
        }
    };
})();
