//=============================================================================
// PersistentData.js
//=============================================================================

/*:
* @plugindesc Piver Persistent Data
*
* @param Auto load on create
* @desc Auto load the persistent data when creating a new game
* @type boolean
* @default true
*
* @param Auto load on load
* @desc Auto load the persistent data when loading a saved game
* @type boolean
* @default true
*
* @param Persistent switches
* @desc The ID of the persistent switches. Formatting example: 1-3,5-6,8,10-11,13
* @type string
* @default 
*
* @param Auto persist switch
* @desc Persist a switch as soon as it's set
* @type boolean
* @default false
*
* @param Persistent variables
* @desc The ID of the persistent variables. Formatting example: 1-3,5-6,8,10-11,13
* @type string
* @default 
*
* @param Auto persist variable
* @desc Persist a variable as soon as it's set
* @type boolean
* @default false
*
* @param Debug
* @desc Show log in console
* @type boolean
* @default false
*
*/

(function () {

    //=============================================================================
    // PersistManager
    //=============================================================================
    function PersistManager() {
        throw new Error('This is a static class');
    }

    PersistManager.load = function () {
        PersistManager.debug('Loading persistant data...');
        var json;
        var persist = {};
        try {
            json = StorageManager.load('Persistent', json);
        } catch (e) {
            console.error(e);
        }
        if (json) {
            persist = JSON.parse(json);
        }
        PersistManager.loadSwitches(persist.switches || []);
        PersistManager.loadVariables(persist.variables || []);
    };

    PersistManager.loadSwitches = function (switches) {
        var tmp = autoPersistSwitch;
        autoPersistSwitch = false;
        persistentSwitches.forEach(function (s) {
            $gameSwitches.setValue(s, switches[s]);
        });
        autoPersistSwitch = tmp;
        PersistManager.debug(`${persistentSwitches.length} persistant switches loaded`);
    };

    PersistManager.loadVariables = function (variables) {
        var tmp = autoPersistVariable;
        autoPersistVariable = false;
        persistentVariables.forEach(function (v) {
            $gameVariables.setValue(v, variables[v]);
        });
        autoPersistVariable = tmp;
        PersistManager.debug(`${persistentVariables.length} persistant variables loaded`);
    };

    PersistManager.save = function () {
        PersistManager.debug('Saving persistant data...');
        StorageManager.save('Persistent', JSON.stringify(this.makeData()));
        PersistManager.debug('Persistant data saved');
    };

    PersistManager.makeData = function () {
        return {
            switches: persistentSwitches.reduce(function (acc, curr) {
                acc[curr] = $gameSwitches.value(curr);
                return acc;
            }, {}),
            variables: persistentVariables.reduce(function (acc, curr) {
                acc[curr] = $gameVariables.value(curr);
                return acc;
            }, {})
        };
    };

    PersistManager.resolve = function (path) {
        if (!path) {
            return [];
        }
        var set = new Set();
        path.split(',').forEach(function (range) {
            var bounds = range.split('-');
            if (bounds.length === 1) {
                var id = parseInt(bounds[0]);
                if (isNaN(id) || id <= 0) {
                    PersistManager.debug(`Ignoring invalid range: ${range}`);
                    return;
                }
                set.add(id);
                return;
            }
            if (bounds.length !== 2 || bounds[0] > bounds[1]) {
                PersistManager.debug(`Ignoring invalid range: ${range}`);
                return;
            }
            var start = parseInt(bounds[0]);
            var end = parseInt(bounds[1]);
            if (isNaN(start) || start <= 0 || isNaN(end) || end <= 0) {
                PersistManager.debug(`Ignoring invalid range: ${range}`);
                return;
            }
            for (var i = start; i <= end; i++) {
                set.add(i);
            }
        });
        var ids = [];
        set.forEach(function (id) { ids.push(id); })
        return ids;
    };

    PersistManager.debug = function (message) {
        if (debug) {
            console.log(message);
        }
    };

    //=============================================================================
    // PluginManager
    //=============================================================================
    var parameters = PluginManager.parameters('PersistentData');
    var autoLoadOnCreate = parameters['Auto load on create'] !== 'false';
    var autoLoadOnLoad = parameters['Auto load on load'] !== 'false';
    var persistentSwitches = PersistManager.resolve(parameters['Persistent switches'] || '');
    var autoPersistSwitch = parameters['Auto persist switch'] === 'true';
    var persistentVariables = PersistManager.resolve(parameters['Persistent variables'] || '');
    var autoPersistVariable = parameters['Auto persist variable'] === 'true';
    var debug = parameters['Debug'] === 'true';

    //=============================================================================
    // Game_Interpreter
    //=============================================================================
    var Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
    Game_Interpreter.prototype.pluginCommand = function (command, args) {
        Game_Interpreter_pluginCommand.call(this, command, args);
        if (command === 'Persistent') {
            switch (args[0]) {
                case 'Save':
                    PersistManager.save(parameters);
                    break;
                case 'Load':
                    PersistManager.load(parameters);
                    break;
            }
        }
    };

    //=============================================================================
    // Game_Switches
    //=============================================================================
    var Game_Switches_setValue = Game_Switches.prototype.setValue;
    Game_Switches.prototype.setValue = function (switchId, value, save) {
        Game_Switches_setValue.call(this, switchId, value);
        if (autoPersistSwitch) {
            PersistManager.save();
        }
    };

    //=============================================================================
    // Game_Variables
    //=============================================================================
    var Game_Variables_setValue = Game_Variables.prototype.setValue;
    Game_Variables.prototype.setValue = function (varId, value, save) {
        Game_Variables_setValue.call(this, varId, value);
        if (autoPersistVariable) {
            PersistManager.save();
        }
    };

    //=============================================================================
    // StorageManager
    //=============================================================================
    var StorageManager_localFilePath = StorageManager.localFilePath;
    StorageManager.localFilePath = function (savefileId) {
        if (savefileId === 'Persistent') {
            return StorageManager.localFileDirectoryPath() + 'persistent.rpgsave';
        }
        return StorageManager_localFilePath.call(this, savefileId);
    };

    //=============================================================================
    // DataManager
    //=============================================================================
    var DataManager_createGameObjects = DataManager.createGameObjects;
    DataManager.createGameObjects = function () {
        DataManager_createGameObjects.call(this);
        if (autoLoadOnCreate) {
            PersistManager.load();
        }
    };

    var DataManager_extractSaveContents = DataManager.extractSaveContents;
    DataManager.extractSaveContents = function (contents) {
        DataManager_extractSaveContents.call(this, contents);
        if (autoLoadOnLoad) {
            PersistManager.load();
        }
    };
})();